import { Relations, createTableRelationsHelpers, is } from "drizzle-orm";
import { SQLiteTable } from "drizzle-orm/sqlite-core";
import {
  GraphQLError,
  GraphQLInt,
  GraphQLList,
  GraphQLNonNull,
  Kind
} from "graphql";
import {
  extractFilters,
  extractOrderBy,
  extractRelationsParams,
  extractSelectedColumnsFromNode,
  extractSelectedColumnsSQLFormat,
  generateTableTypes
} from "./common";
import { camelize, pascalize } from "../case-ops";
import {
  remapFromGraphQLArrayInput,
  remapFromGraphQLSingleInput,
  remapToGraphQLArrayOutput,
  remapToGraphQLSingleOutput
} from "../data-mappers";
const generateSelectArray = (db, tableName, table, relations, orderArgs, filterArgs) => {
  const queryName = `${camelize(tableName)}`;
  const queryBase = db.query[tableName];
  if (!queryBase)
    throw new Error(
      `Table ${tableName} not found in drizzle instance. Did you forget to pass schema to drizzle constructor?`
    );
  const queryArgs = {
    offset: {
      type: GraphQLInt
    },
    limit: {
      type: GraphQLInt
    },
    orderBy: {
      type: orderArgs
    },
    where: {
      type: filterArgs
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      try {
        const { offset, limit, orderBy, where } = args;
        const tableSelection = info.operation.selectionSet.selections.find(
          (e) => e.kind === Kind.FIELD && e.name.value === queryName
        );
        const query = queryBase.findMany({
          columns: extractSelectedColumnsFromNode(tableSelection, table),
          offset,
          limit,
          orderBy: orderBy ? extractOrderBy(table, orderBy) : void 0,
          where: where ? extractFilters(table, tableName, where) : void 0,
          with: relations ? extractRelationsParams(relations, tableSelection, `${pascalize(tableName)}SelectItem`, info) : void 0
        });
        const result = await query;
        return remapToGraphQLArrayOutput(result, relations);
      } catch (e) {
        if (typeof e === "object" && typeof e.message === "string")
          throw new GraphQLError(e.message);
        throw e;
      }
    },
    args: queryArgs
  };
};
const generateSelectSingle = (db, tableName, table, relations, orderArgs, filterArgs) => {
  const queryName = `${camelize(tableName)}Single`;
  const queryBase = db.query[tableName];
  if (!queryBase)
    throw new Error(
      `Table ${tableName} not found in drizzle instance. Did you forget to pass schema to drizzle constructor?`
    );
  const queryArgs = {
    offset: {
      type: GraphQLInt
    },
    orderBy: {
      type: orderArgs
    },
    where: {
      type: filterArgs
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      try {
        const { offset, orderBy, where } = args;
        const tableSelection = info.operation.selectionSet.selections.find(
          (e) => e.kind === Kind.FIELD && e.name.value === queryName
        );
        const query = queryBase.findFirst({
          columns: extractSelectedColumnsFromNode(tableSelection, table),
          offset,
          orderBy: orderBy ? extractOrderBy(table, orderBy) : void 0,
          where: where ? extractFilters(table, tableName, where) : void 0,
          with: relations ? extractRelationsParams(relations, tableSelection, `${pascalize(tableName)}SelectItem`, info) : void 0
        });
        const result = await query;
        if (!result)
          return void 0;
        return remapToGraphQLSingleOutput(result, relations);
      } catch (e) {
        if (typeof e === "object" && typeof e.message === "string")
          throw new GraphQLError(e.message);
        throw e;
      }
    },
    args: queryArgs
  };
};
const generateInsertArray = (db, tableName, table, baseType) => {
  const queryName = `insertInto${pascalize(tableName)}`;
  const queryArgs = {
    values: {
      type: new GraphQLNonNull(new GraphQLList(new GraphQLNonNull(baseType)))
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      try {
        const input = remapFromGraphQLArrayInput(args.values, table);
        if (!input.length)
          throw new GraphQLError("No values were provided!");
        const columns = extractSelectedColumnsSQLFormat(info, queryName, table);
        const result = await db.insert(table).values(input).returning(columns).onConflictDoNothing();
        return remapToGraphQLArrayOutput(result);
      } catch (e) {
        if (typeof e === "object" && typeof e.message === "string")
          throw new GraphQLError(e.message);
        throw e;
      }
    },
    args: queryArgs
  };
};
const generateInsertSingle = (db, tableName, table, baseType) => {
  const queryName = `insertInto${pascalize(tableName)}Single`;
  const queryArgs = {
    values: {
      type: new GraphQLNonNull(baseType)
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      try {
        const input = remapFromGraphQLSingleInput(args.values, table);
        const columns = extractSelectedColumnsSQLFormat(info, queryName, table);
        const result = await db.insert(table).values(input).returning(columns).onConflictDoNothing();
        if (!result[0])
          return void 0;
        return remapToGraphQLSingleOutput(result[0]);
      } catch (e) {
        if (typeof e === "object" && typeof e.message === "string")
          throw new GraphQLError(e.message);
        throw e;
      }
    },
    args: queryArgs
  };
};
const generateUpdate = (db, tableName, table, setArgs, filterArgs) => {
  const queryName = `update${pascalize(tableName)}`;
  const queryArgs = {
    set: {
      type: new GraphQLNonNull(setArgs)
    },
    where: {
      type: filterArgs
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      try {
        const { where, set } = args;
        const columns = extractSelectedColumnsSQLFormat(info, queryName, table);
        const input = remapFromGraphQLSingleInput(set, table);
        if (!Object.keys(input).length)
          throw new GraphQLError("Unable to update with no values specified!");
        let query = db.update(table).set(input);
        if (where) {
          const filters = extractFilters(table, tableName, where);
          query = query.where(filters);
        }
        query = query.returning(columns);
        const result = await query;
        return remapToGraphQLArrayOutput(result);
      } catch (e) {
        if (typeof e === "object" && typeof e.message === "string")
          throw new GraphQLError(e.message);
        throw e;
      }
    },
    args: queryArgs
  };
};
const generateDelete = (db, tableName, table, filterArgs) => {
  const queryName = `deleteFrom${pascalize(tableName)}`;
  const queryArgs = {
    where: {
      type: filterArgs
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      try {
        const { where } = args;
        const columns = extractSelectedColumnsSQLFormat(info, queryName, table);
        let query = db.delete(table);
        if (where) {
          const filters = extractFilters(table, tableName, where);
          query = query.where(filters);
        }
        query = query.returning(columns);
        const result = await query;
        return remapToGraphQLArrayOutput(result);
      } catch (e) {
        if (typeof e === "object" && typeof e.message === "string")
          throw new GraphQLError(e.message);
        throw e;
      }
    },
    args: queryArgs
  };
};
const generateSchemaData = (db, schema) => {
  const rawSchema = schema;
  const schemaEntries = Object.entries(rawSchema);
  const tables = Object.fromEntries(schemaEntries.filter(([key, value]) => is(value, SQLiteTable)));
  if (!tables || !Object.keys(tables).length)
    throw new Error(
      `Unable to extract tables from drizzle instance.
Did you forget to pass tables to graphql schema constructor?`
    );
  const relations = Object.fromEntries(
    schemaEntries.filter(([key, value]) => is(value, Relations)).map(([key, value]) => [
      Object.entries(tables).find(
        ([tableName, tableValue]) => tableValue === value.table
      )[0],
      value
    ]).map(([tableName, relValue]) => [
      tableName,
      relValue.config(createTableRelationsHelpers(tables[tableName]))
    ])
  );
  const queries = {};
  const mutations = {};
  const gqlSchemaTypes = Object.fromEntries(
    Object.entries(tables).map(([tableName, table]) => [
      tableName,
      generateTableTypes(tableName, table, true, relations[tableName])
    ])
  );
  const inputs = {};
  const outputs = {};
  for (const [tableName, tableTypes] of Object.entries(gqlSchemaTypes)) {
    const { insertInput, updateInput, tableFilters, tableOrder } = tableTypes.inputs;
    const { selectSingleOutput, selectArrOutput, singleTableItemOutput, arrTableItemOutput } = tableTypes.outputs;
    const selectArrGenerated = generateSelectArray(
      db,
      tableName,
      schema[tableName],
      relations[tableName],
      tableOrder,
      tableFilters
    );
    const selectSingleGenerated = generateSelectSingle(
      db,
      tableName,
      schema[tableName],
      relations[tableName],
      tableOrder,
      tableFilters
    );
    const insertArrGenerated = generateInsertArray(db, tableName, schema[tableName], insertInput);
    const insertSingleGenerated = generateInsertSingle(db, tableName, schema[tableName], insertInput);
    const updateGenerated = generateUpdate(
      db,
      tableName,
      schema[tableName],
      updateInput,
      tableFilters
    );
    const deleteGenerated = generateDelete(db, tableName, schema[tableName], tableFilters);
    queries[selectArrGenerated.name] = {
      type: selectArrOutput,
      args: selectArrGenerated.args,
      resolve: selectArrGenerated.resolver
    };
    queries[selectSingleGenerated.name] = {
      type: selectSingleOutput,
      args: selectSingleGenerated.args,
      resolve: selectSingleGenerated.resolver
    };
    mutations[insertArrGenerated.name] = {
      type: arrTableItemOutput,
      args: insertArrGenerated.args,
      resolve: insertArrGenerated.resolver
    };
    mutations[insertSingleGenerated.name] = {
      type: singleTableItemOutput,
      args: insertSingleGenerated.args,
      resolve: insertSingleGenerated.resolver
    };
    mutations[updateGenerated.name] = {
      type: arrTableItemOutput,
      args: updateGenerated.args,
      resolve: updateGenerated.resolver
    };
    mutations[deleteGenerated.name] = {
      type: arrTableItemOutput,
      args: deleteGenerated.args,
      resolve: deleteGenerated.resolver
    };
    [insertInput, updateInput, tableFilters, tableOrder].forEach((e) => inputs[e.name] = e);
    outputs[selectSingleOutput.name] = selectSingleOutput;
    outputs[singleTableItemOutput.name] = singleTableItemOutput;
  }
  return { queries, mutations, inputs, types: outputs };
};
export {
  generateSchemaData
};
//# sourceMappingURL=sqlite.js.map